@@ -193,13 +193,15 @@ class TailsInstallerThread(threading.Thread):
                 if not self.live.can_read_partition_table():
                     self.live.log.info('Clearing unreadable partition table.')
                     self.live.clear_all_partition_tables()
-                if self.parent.force_reinstall:
-                    parent = _to_unicode(self.live.drive['parent'])
-                    self.rescan_devices(force_partitions=False)
-                    if parent in self.live.drives and \
-                       'device' in self.live.drives[parent]:
-                        self.live.drive = self.live.drives[parent]['device']
-                        self.live.save_full_drive()
+                # If there is a partition already in place and we need
+                # to reinstall, then we need to change the selected
+                # device for its saved parent, if it exists
+                if self.parent.force_reinstall and \
+                   self.live.drive['parent_data'] is not None:
+                    parent_data = self.live.drive['parent_data']
+                    self.live.drives[parent_data['device']] = parent_data
+                    self.live.drive = parent_data['device']
+                    self.live.save_full_drive()
                 self.live.partition_device()
                 self.rescan_devices(force_partitions=True)
                 self.live.switch_drive_to_system_partition()
@@ -394,6 +396,7 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
         self.__radio_button_source_iso = builder.get_object('radio_button_source_iso')
         self.__radio_button_source_device = builder.get_object('radio_button_source_device')
         self.__button_force_reinstall = builder.get_object('check_force_reinstall')
+        self.__help_link = builder.get_object('help_link')
 
         self.add(self.__box_installer)
         builder.connect_signals(self)
@@ -445,7 +448,6 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
             return
         self.force_reinstall = True
         self.opts.partition = True
-        self.live.force_reinstall = True
         self.__button_force_reinstall.set_visible(False)
         self.on_start_clicked(button)
 
@@ -454,7 +456,6 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
 
     def on_target_changed(self, combobox_target):
         # get selected device
-        self.__button_force_reinstall.set_visible(False)
         drive = self.get_selected_drive()
         if drive == None:
            return
@@ -463,15 +464,24 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
         if self.live.device_can_be_upgraded(device):
             self.opts.partition = False
             self.force_reinstall = False
-            self.live.force_reinstall = False
             self.__button_start.set_label(_('Upgrade'))
-            self.__button_force_reinstall.set_visible(True)
+            self.__help_link.set_label(
+                                     _('Manual Upgrade Instructions'))
+            self.__help_link.set_uri(
+                        _('https://tails.boum.org/upgrade/'))
+            if device['is_device_big_enough_for_reinstall']:
+                self.__button_force_reinstall.set_visible(True)
+            else:
+                self.__button_force_reinstall.set_visible(False)
         else:
             self.opts.partition = True
             self.force_reinstall = True
-            self.live.force_reinstall = False
             self.__button_start.set_label(_('Install'))
             self.__button_force_reinstall.set_visible(False)
+            self.__help_link.set_label(
+                                     _('Installation Instructions'))
+            self.__help_link.set_uri(
+                        _('https://tails.boum.org/install/'))
 
     def get_device_pretty_name(self, device):
         size = _format_bytes_in_gb(device['parent_size']
@@ -520,8 +530,6 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
     def update_start_button(self):
         if self.source_available and self.target_available:
             self.__button_start.set_sensitive(True)
-            if not self.opts.partition:
-                self.__button_force_reinstall.set_visible(True)
         else:
             self.__button_start.set_sensitive(False)
 
@@ -688,17 +696,20 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
         dialog.run()
         self.close()
 
-    def show_confirmation_dialog(self, title, message, warning):
+    def show_confirmation_dialog(self, title, message, warning,
+                                 label_string=_("Install")):
         if (warning):
             buttons=Gtk.ButtonsType.OK
         else:
-            buttons=Gtk.ButtonsType.YES_NO
+            buttons=Gtk.ButtonsType.NONE
         dialog = Gtk.MessageDialog(parent=self,
                                    flags=Gtk.DialogFlags.DESTROY_WITH_PARENT,
                                    message_type=Gtk.MessageType.QUESTION,
                                    buttons=buttons,
                                    message_format=title)
         dialog.format_secondary_text(message)
+        dialog.add_button(_("Cancel"), Gtk.ResponseType.CANCEL);
+        dialog.add_button(label_string, Gtk.ResponseType.YES);
         reply = dialog.run()
         dialog.hide()
         if (reply == Gtk.ResponseType.YES):
@@ -746,9 +757,12 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
                           {'vendor': self.live.drive['vendor'],
                            'model':  self.live.drive['model'],
                            'device': self.live.drive['device'],
-                           'size':   _format_bytes_in_gb(self.live.drive['size'])
+                           'size':   _format_bytes_in_gb(self.live.drive['parent_size']
+                                    if self.live.drive['parent_size']
+                                    else self.live.drive['size'])
                           },
-                          False):
+                          False,
+                          label_string=_("Install")):
                     self.confirmed = True
                 else:
                     return
@@ -770,7 +784,8 @@ class TailsInstallerWindow(Gtk.ApplicationWindow):
                 'description': description,
                 'persistence_message': persistence_message,
             }
-            if self.show_confirmation_dialog(_("Confirm the target USB stick"), msg, False):
+            if self.show_confirmation_dialog(_("Confirm the target USB stick"), \
+               msg, False, label_string=_("Upgrade")):
                 # The user has confirmed that they wish to overwrite their
                 # existing Live OS.  Here we delete it first, in order to
                 # accurately calculate progress.
